using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Hardcopy_Export(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Hardcopy_Export
              * This sample will show you how to:
              * - Import an HSF file
              * - Retrieve the size of the window at runtime
              * - Export a 2D-PDF and PostScript file
              * - Check whether the Hardcopy Export completed successfully
              * 
              * Read more about the topics covered in this sample in our Programming Guide 
              * - section x.x:	 */

            HPS.SegmentKey description = modelKey.Subsegment();

            HPS.TextKit textKit = new HPS.TextKit();
            textKit.SetSize(0.02f, Text.SizeUnits.SubscreenRelative);
            HPS.Point[] point = { new HPS.Point(-0.95f, -0.70f, 10), new HPS.Point(0.95f, -0.70f, 10), new HPS.Point(0, -1, 10) };
            textKit.SetRegion(point, Text.RegionAlignment.Center, Text.RegionFitting.Left, false, false, true);
            textKit.SetBackground(true, "rounded box");
            textKit.SetAlignment(HPS.Text.Alignment.BottomLeft);
            HPS.MaterialMappingKit matMapKit = new HPS.MaterialMappingKit();
            matMapKit.SetFaceColor(new RGBAColor(1, 1, 1));
            matMapKit.SetEdgeColor(new RGBAColor(0, 0, 0));

            /* This next block of code shows you how to load an HSF file
	         * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	         * 2. Use the Stream.File.Import method to load the desired file, with the options you specified
	         * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	         * 4. Use a try-catch block to capture any Stream IO exceptions */
            HPS.Stream.ImportNotifier notifier = new HPS.Stream.ImportNotifier();
            HPS.SegmentKey geometry = modelKey.Subsegment();

            try
            {
                HPS.Stream.ImportOptionsKit importOptionsKit = new HPS.Stream.ImportOptionsKit();
                importOptionsKit.SetSegment(geometry);
                string filename = INPUT_FOLDER + "/conrod.hsf";

                notifier = HPS.Stream.File.Import(filename, importOptionsKit);
                notifier.Wait();
            }
            catch (HPS.IOException)
            {
                /* catch Stream I/O Errors here */
            }

            view.FitWorld();
            view.Update();

            /* This block of code shows you how to export a 2D-PDF and a Postscript file representing the scene
	         * 1. Use a Hardcopy.File.ExportOptionKit object to set the options needed for the export 
	         * 2. To obtain the size of a window, get its WindowInfoControl object and use its ShowPhysicalSize method
	         * 3. Setting WYSIWYG on the export kit matches the model on screen to the hardcopy output exactly
	         * 4. The Hardcopy.File:Export method is where the format to export to is specified
	         * 5. Hardcopy.File.Export returns a Hardcopy.IOResult object which can be checked to see if the
	         *		export operation completed successfully. */
            HPS.IOResult PDFResult = HPS.IOResult.Failure, postScriptResult = HPS.IOResult.Failure;
            try
            {
                string file_out = OUTPUT_FOLDER + "/hardcopy_export.pdf";
                HPS.Hardcopy.File.ExportOptionsKit exportOptionsKit = new HPS.Hardcopy.File.ExportOptionsKit();
                float width, height;

                wk.GetWindowInfoControl().ShowPhysicalSize(out width, out height);
                exportOptionsKit.SetWYSIWYG(true);
                exportOptionsKit.SetSize(width, height, HPS.Hardcopy.SizeUnits.Centimeters);
                exportOptionsKit.SetResolution(100, HPS.Hardcopy.ResolutionUnits.DPCM);



                PDFResult = HPS.Hardcopy.File.Export(file_out, HPS.Hardcopy.File.Driver.PDF, wk, exportOptionsKit);
                file_out = OUTPUT_FOLDER + "/hardcopy_export.ps";
                postScriptResult = HPS.Hardcopy.File.Export(file_out, HPS.Hardcopy.File.Driver.Postscript, wk, exportOptionsKit);
            }
            catch (HPS.IOException)
            {
                //handle Hardcopy exception
            }


            /*Here the Material Kit defined previously is attached to the description subsegment
             *Edges are set to visible
             *The matMapKit is attached to the description subsegment to ensure its style does not affect the model Key Material Mapping Style*/

            description.GetVisibilityControl().SetEdges(true);
            description.SetMaterialMapping(matMapKit);

            /* This block of code checks the result of the two export operations completed above
	         * and prints out a message on the screen informing the user whether they completed
	         * successfully or not*/

            string resultDescription;
            if (PDFResult == HPS.IOResult.Success)
                resultDescription = "PDF Export completed successfully.";
            else
                resultDescription = "PDF Export FAILED.";

            if (postScriptResult == HPS.IOResult.Success)
                resultDescription += "\nPostScript Export completed successfully.";
            else
                resultDescription += "\nPostScript Export FAILED.";

            resultDescription += "\nYou can find the exported files in this folder:\n   " + OUTPUT_FOLDER;
            
            textKit.SetText(resultDescription);

            description.InsertText(textKit);
        }
    }
}
